<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class Fm00004001TransferirTarefasFactory
{
    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getOperadorDestino($parametros, $listaLocais, $filtro)
    {
        $sql = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR, FCM_RELACHABOPER.NRHABILIDADE
                    FROM OPERADOR
                    JOIN FCM_FUNCIONARIOSHIERARQ
                    ON FCM_FUNCIONARIOSHIERARQ.CDOPERADOR = OPERADOR.CDOPERADOR
                    JOIN FCM_HIERARQUIAFUNC
                    ON FCM_HIERARQUIAFUNC.NRHIERARQUIAFUNC = FCM_FUNCIONARIOSHIERARQ.NRHIERARQUIAFUNC
                    JOIN FCM_RELACHABOPER
                    ON FCM_RELACHABOPER.CDOPERADOR = OPERADOR.CDOPERADOR
                    WHERE FCM_HIERARQUIAFUNC.NRLOCALCLIENTE IN (SELECT NRLOCALCLIENTE
                                            FROM FCM_LOCALCLI
                                            START WITH NRLOCALCLIENTE IN ($listaLocais)
                                            CONNECT BY PRIOR NRLOCALCLIENTEPAI = NRLOCALCLIENTE) 
                    AND FCM_RELACHABOPER.NRHABILIDADE = :NRHABILIDADE
                    $filtro
                    ORDER BY OPERADOR.NMOPERADOR";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function transferirTarefa($parametros)
    {
        $sql = "UPDATE FCM_CHECKLISTAGEND
                    SET CDOPERADOR = :CDOPERADOR,
                        DTULTATU = SYSDATE,
                        CDOPERULTATU = :CDOPERULTATU,
                        NRORGULTATU = :NRORGULTATU,
                        DTHORAAGEND = :DTHORAAGEND,
                        DTHRFINALAGEND = :DTHRFINALAGEND
               WHERE NRCHECKLISTAGEND =  :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getTarefasPorOperador($operador)
    {
        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.NRFUNCIONARIO, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, 
                        TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND,'DD/MM/YYYY HH24:MI') AS DTHORAAGEND, FCM_CHECKLISTAGEND.IDATIVO, 
                       FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, TO_CHAR(FCM_CHECKLISTAGEND.DTHRFINALAGEND, 'DD/MM/YYYY HH24:MI') AS DTHRFINALAGEND, 
                       FCM_CHECKLISTAGEND.CDOPERADOR, FCM_RELACHABOPER.NRHABILIDADE, FCM_CHECKLISTAGEND.NRPRIORIDADETAREFA, FCM_PRIORIDADETAREFA.IDPRIORIDADE
                FROM FCM_CHECKLISTAGEND
                JOIN FCM_RELACHABOPER
                    ON FCM_RELACHABOPER.CDOPERADOR = FCM_CHECKLISTAGEND.CDOPERADOR
                LEFT JOIN FCM_PRIORIDADETAREFA
                    ON FCM_CHECKLISTAGEND.NRPRIORIDADETAREFA = FCM_PRIORIDADETAREFA.NRPRIORIDADETAREFA
                WHERE FCM_CHECKLISTAGEND.IDATIVO = 'S'
                AND FCM_CHECKLISTAGEND.CDOPERADOR =  $operador
                AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND NOT IN ('4', '3')";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    /* TRANSFEERÊNCIAS DE TAREFAS AUTOMÁTICAS */

    public function getTarefasAtrasadas($entityManager)
    {
        $sql = "SELECT 
                    FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, 
                    (FCM_CHECKLISTAGEND.DTHRFINALAGEND - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24 * 60 AS DURACAO_MINUTOS,
                    FCM_CHECKLISTAGEND.NRLOCALCLIENTE,
                    FCM_CHECKLISTAGEND.CDOPERADOR,
                    FCM_PRIORIDADETAREFA.ATRASOPERMITIDO,
                    FCM_RELACHABOPER.NRHABILIDADE,
                    FCM_PRIORIDADETAREFA.NRPRIORIDADETAREFA,
                    (TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 1, 2)) * 60 +
                    TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 4, 2))) AS ATRASOPERMITIDO_MINUTOS,  
                    (ROUND((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24 * 60)) AS TEMPOATRASADO,
                    TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND,'DD/MM/YYYY HH24:MI') AS DTHORAAGEND,
                    TO_CHAR(FCM_CHECKLISTAGEND.DTHRFINALAGEND,'DD/MM/YYYY HH24:MI') AS DTHRFINALAGEND
                FROM  FCM_CHECKLISTAGEND
                  JOIN  FCM_PRIORIDADETAREFA
                    ON  FCM_PRIORIDADETAREFA.NRPRIORIDADETAREFA = FCM_CHECKLISTAGEND.NRPRIORIDADETAREFA
        LEFT JOIN FCM_RELACHABOPER
                    ON FCM_RELACHABOPER.CDOPERADOR = FCM_CHECKLISTAGEND.CDOPERADOR
                WHERE  
                    (ROUND((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24 * 60)) >
                    (TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 1, 2)) * 60 +
                    TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 4, 2)))
                    ORDER BY FCM_PRIORIDADETAREFA.NRPRIORIDADETAREFA";
             
        return $entityManager->getConnection()->fetchAll($sql);
    }

    public function getListaOperadores($parametros, $entityManager, $paramHabilidade, $listaLocais)
    {
        $sql = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR, FCM_RELACHABOPER.NRHABILIDADE
                    FROM OPERADOR
                    JOIN FCM_FUNCIONARIOSHIERARQ
                    ON FCM_FUNCIONARIOSHIERARQ.CDOPERADOR = OPERADOR.CDOPERADOR
                    JOIN FCM_HIERARQUIAFUNC
                    ON FCM_HIERARQUIAFUNC.NRHIERARQUIAFUNC = FCM_FUNCIONARIOSHIERARQ.NRHIERARQUIAFUNC
                    LEFT JOIN FCM_RELACHABOPER
                    ON FCM_RELACHABOPER.CDOPERADOR = OPERADOR.CDOPERADOR
                    WHERE FCM_HIERARQUIAFUNC.NRLOCALCLIENTE IN (SELECT NRLOCALCLIENTE
                                            FROM FCM_LOCALCLI
                                            START WITH NRLOCALCLIENTE IN ($listaLocais)
                                            CONNECT BY PRIOR NRLOCALCLIENTEPAI = NRLOCALCLIENTE)
                    AND OPERADOR.CDOPERADOR <> :CDOPERADOR
                    $paramHabilidade
                    ORDER BY OPERADOR.NMOPERADOR";
        
        return $entityManager->getConnection()->fetchAll($sql, $parametros);
    }

    public function apiGetParametrizacoes($entityManager)
    {
        $sql = "SELECT  IDVALIDACRIATAREFA, IDUTILIZAHABILIDADE, IDVALIDACONFLITOURG
                        FROM FCM_PARAMGERAL";
        
        $dados = $entityManager->getConnection()->fetchAll($sql);
        return $dados[0];
    }

    public function apiInsertLogTransferencia($parametros, $entityManager) {

        $sql = "INSERT INTO FCM_LOGTRANSFAGEND (NRLOGTRANSFAGEND, NRCHECKLISTAGEND, DTHORAAGENDORIG, DTHORAAGENDDESTINO, NRORG, 
                                                                                        IDATIVO, DTINCLUSAO, DTULTATU, CDOPERADORORIGEM, CDOPERADORDESTINO)
                    VALUES (:NRLOGTRANSFAGEND, :NRCHECKLISTAGEND, :DTHORAAGENDORIG, SYSDATE, :NRORG, 
                              'S', SYSDATE, SYSDATE, :CDOPERADORORIGEM, :CDOPERADORDESTINO)";
        
        $entityManager->getConnection()->executeStatement($sql, $parametros);
    }

    public function apiTransferirTarefa($parametros, $entityManager)
    {
        $sql = "UPDATE FCM_CHECKLISTAGEND
                    SET CDOPERADOR = :CDOPERADOR,
                        DTULTATU = SYSDATE,
                        DTHORAAGEND = SYSDATE,
                        DTHRFINALAGEND = SYSDATE + NUMTODSINTERVAL(:DURACAO_MINUTOS, 'MINUTE')
               WHERE NRCHECKLISTAGEND =  :NRCHECKLISTAGEND";

        $entityManager->getConnection()->executeStatement($sql, $parametros);
    }

    public function apiGetTarefasPorOperador($parametros, $entityManager)
    {
        $sql = "SELECT FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NRLOCALCLIENTE, FCM_CHECKLISTAGEND.NRCHECKLIST, FCM_CHECKLISTAGEND.NRFUNCIONARIO, FCM_CHECKLISTAGEND.IDSITUACAOAGEND, 
                        TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND,'DD/MM/YYYY HH24:MI') AS DTHORAAGEND, FCM_CHECKLISTAGEND.IDATIVO, 
                       FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, TO_CHAR(FCM_CHECKLISTAGEND.DTHRFINALAGEND, 'DD/MM/YYYY HH24:MI') AS DTHRFINALAGEND, 
                       FCM_CHECKLISTAGEND.CDOPERADOR, FCM_RELACHABOPER.NRHABILIDADE, FCM_CHECKLISTAGEND.NRPRIORIDADETAREFA, FCM_PRIORIDADETAREFA.IDPRIORIDADE
                FROM FCM_CHECKLISTAGEND
                JOIN FCM_RELACHABOPER
                    ON FCM_RELACHABOPER.CDOPERADOR = FCM_CHECKLISTAGEND.CDOPERADOR
                LEFT JOIN FCM_PRIORIDADETAREFA
                    ON FCM_CHECKLISTAGEND.NRPRIORIDADETAREFA = FCM_PRIORIDADETAREFA.NRPRIORIDADETAREFA
                WHERE FCM_CHECKLISTAGEND.IDATIVO = 'S'
                AND FCM_CHECKLISTAGEND.CDOPERADOR = :CDOPERADOR
                AND FCM_CHECKLISTAGEND.IDSITUACAOAGEND NOT IN ('4', '3')
                AND TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) = TRUNC(TO_DATE(:DTHORAAGEND))";

        return $entityManager->getConnection()->fetchAll($sql, $parametros);
    }

    public function apiGetHistoricoTransferencia($parametros)
    {
        $sql = "SELECT 
                    FCM_LOGTRANSFAGEND.NRCHECKLISTAGEND, 
                    FCM_CHECKLISTAGEND.NMCHECKLISTAGEND, 
                    TO_CHAR(FCM_CHECKLISTAGEND.DTHORAAGEND, 'DD/MM/YYYY HH24:MI') AS DTHORAAGEND, 
                    ORIGEM.NMOPERADOR,
                    DESTINO.NMOPERADOR AS FUNCDESTINO, 
                    TO_CHAR(FCM_LOGTRANSFAGEND.DTINCLUSAO, 'DD/MM/YYYY HH24:MI') AS DTINCLUSAO, 
                    NVL(RESPDESTINOONSAVEL.NMOPERADOR, 'Transferência automática') AS RESPONSAVEL,
                    CASE
                        WHEN (TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 1, 2)) * 60 +
                            TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 4, 2))) >= 60 THEN
                            FLOOR((TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 1, 2)) * 60 +
                                TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 4, 2))) / 60) || ' horas ' || 
                            MOD((TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 1, 2)) * 60 +
                                TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 4, 2))), 60) || ' minutos'
                        ELSE
                            (TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 1, 2)) * 60 +
                            TO_NUMBER(SUBSTR(FCM_PRIORIDADETAREFA.ATRASOPERMITIDO, 4, 2))) || ' minutos'
                    END AS ATRASOPERMITIDO,  
                    CASE
                        WHEN SYSDATE < FCM_CHECKLISTAGEND.DTHORAAGEND THEN ' '
                        WHEN TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) < 1 THEN 
                            GREATEST(FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24), 0) || ' horas ' || 
                            GREATEST(FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24 * 60) - (FLOOR((SYSDATE - FCM_CHECKLISTAGEND.DTHORAAGEND) * 24) * 60), 0) || ' minutos'
                        WHEN TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) = 1 THEN
                            TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) || ' dia'
                        ELSE 
                            TRUNC(SYSDATE) - TRUNC(FCM_CHECKLISTAGEND.DTHORAAGEND) || ' dias'
                    END AS TEMPOATRASADO
                FROM 
                    FCM_LOGTRANSFAGEND
                LEFT JOIN 
                    FCM_CHECKLISTAGEND ON FCM_LOGTRANSFAGEND.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                LEFT JOIN 
                    OPERADOR ORIGEM ON FCM_LOGTRANSFAGEND.CDOPERADORORIGEM = ORIGEM.CDOPERADOR
                LEFT JOIN 
                    OPERADOR DESTINO ON FCM_LOGTRANSFAGEND.CDOPERADORDESTINO = DESTINO.CDOPERADOR
                LEFT JOIN 
                    OPERADOR RESPDESTINOONSAVEL ON FCM_LOGTRANSFAGEND.CDOPERREALIZATRANSF = RESPDESTINOONSAVEL.CDOPERADOR
                LEFT JOIN 
                    FCM_PRIORIDADETAREFA ON FCM_PRIORIDADETAREFA.NRPRIORIDADETAREFA = FCM_CHECKLISTAGEND.NRPRIORIDADETAREFA
                WHERE  
                    FCM_LOGTRANSFAGEND.NRCHECKLISTAGEND = :NRCHECKLISTAGEND
                ORDER BY  
                    FCM_LOGTRANSFAGEND.DTULTATU DESC";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }
}
