<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class FmtOrdemServFactory
{
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getOrdensServico()
    {
        $sql = "SELECT OSE.NRORDEMSERV, TO_CHAR(OSE.DTORDEMSERV, 'DD/MM/YY') DTORDEMSERV, OSE.IDSITUACAO, EP.NMEQUIPAMENTO,
                CAT.NMCATEGORIAMANUT, TM.NMTIPOMANUTENCAO, P.NMPRIORIDADEMANUT, P.DSCORPRIORIDADE, OSE.NREQUIPAMENTO,
                OSE.NRCATEGORIAMANUT, OSE.NRTIPOMANUTENCAO, OSE.NRPRIORIDADEMANUT, OSE.DSORDEMSERV,
                  NVL((SELECT LISTAGG(FCM_FUNCIONARIO.NMFUNCIONARIO, ',')
                      WITHIN GROUP (ORDER BY FCM_RESPONSAVEISORDEMSERV.NRFUNCIONARIO)
                        FROM FCM_RESPONSAVEISORDEMSERV
                        JOIN FCM_FUNCIONARIO
                          ON FCM_RESPONSAVEISORDEMSERV.NRFUNCIONARIO = FCM_FUNCIONARIO.NRFUNCIONARIO
                       WHERE OSE.NRORDEMSERV = FCM_RESPONSAVEISORDEMSERV.NRORDEMSERV), ' ') NMFUNCIONARIO,
                NVL((SELECT LISTAGG(FCM_ESCOPOMANUT.NMESCOPOMANUT, ',')
                    WITHIN GROUP (ORDER BY FCM_ESCOPOMANUT.NRESCOPOMANUT)
                    FROM FCM_RELESCORDEMSERV
                    JOIN FCM_ESCOPOMANUT
                    ON FCM_RELESCORDEMSERV.NRESCOPOMANUT = FCM_ESCOPOMANUT.NRESCOPOMANUT
                    WHERE OSE.NRORDEMSERV = FCM_RELESCORDEMSERV.NRORDEMSERV), ' ') NMESCOPOMANUT,
                 CASE OSE.IDSITUACAO WHEN '1' THEN 'Pendente'
                    WHEN '2' THEN 'Em Andamento'
                    WHEN '3' THEN 'Concluído'
                    ELSE NULL END AS NMIDSITUACAO
                 FROM FCM_ORDEMSERV OSE
                 JOIN FCM_PRIORIDADEMANUT P
                   ON P.NRPRIORIDADEMANUT = OSE.NRPRIORIDADEMANUT
                 JOIN FCM_EQUIPAMENTO EP
                   ON EP.NREQUIPAMENTO = OSE.NREQUIPAMENTO
                 JOIN FCM_CATEGORIAMANUT CAT
                   ON CAT.NRCATEGORIAMANUT = OSE.NRCATEGORIAMANUT
                 JOIN FCM_TIPOMANUTENCAO TM
                   ON TM.NRTIPOMANUTENCAO = OSE.NRTIPOMANUTENCAO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getComentariosOrdem($parametros)
    {
        $sql = "SELECT COMENT.NRCOMENTORDEMSERV, NVL(FUNC.NMFUNCIONARIO,'Sem nome') NMFUNCIONARIO, TO_CHAR(COMENT.DTCOMENTARIO, 'DD/MM/YYYY') AS DTCOMENTARIO, COMENT.DSCOMENTARIO
                FROM FCM_COMENTORDEMSERV COMENT
                LEFT JOIN FCM_FUNCIONARIO FUNC
                ON COMENT.NRFUNCCOMENTARIO = FUNC.NRFUNCIONARIO
                WHERE COMENT.NRORDEMSERV = :NRORDEMSERV
                ORDER BY DTCOMENTARIO, NRCOMENTORDEMSERV DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function insertOrdemServico($parametros)
    {
        $sql = "INSERT INTO FCM_ORDEMSERV(
                    NRORDEMSERV,        NMORDEMOSERV,       IDSITUACAO,     NREQUIPAMENTO,  NRCATEGORIAMANUT,
                    NRTIPOMANUTENCAO,   NRPRIORIDADEMANUT,  DSORDEMSERV,    DTINCLUSAO,     IDFUNCEXTERNO,
                    NRORG,              DTORDEMSERV
                )
                VALUES(
                    :NRORDEMSERV,       :NMORDEMOSERV,       :IDSITUACAO,    :NREQUIPAMENTO, :NRCATEGORIAMANUT,
                    :NRTIPOMANUTENCAO,  :NRPRIORIDADEMANUT, :DSORDEMSERV,   SYSDATE,        :IDFUNCEXTERNO,
                    :NRORG,             TO_DATE(:DTORDEMSERV, 'DD/MM/YYYY')
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertFotoOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_IMGORDEMSERV (
                    NRIMGORDEMSERV,     NRORDEMSERV,    DSCAMINHO,  NRORG,
                    DTINCLUSAO,         DSTYPE,         NRSIZE
                )
                VALUES (
                    :NRIMGORDEMSERV,    :NRORDEMSERV,   :DSCAMINHO, :NRORG,
                    SYSDATE,            :DSTYPE,        :NRSIZE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function saveComentarioOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_COMENTORDEMSERV (
                    NRCOMENTORDEMSERV,  NRORDEMSERV,    DTCOMENTARIO,   DSCOMENTARIO
                )
                VALUES (
                    :NRCOMENTORDEMSERV, :NRORDEMSERV,   SYSDATE,        :DSCOMENTARIO
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirEscopoRelacionado($parametros)
    {

        $sql = "INSERT INTO FCM_RELESCORDEMSERV (NRRELESCORDEMSERV, NRORDEMSERV, NRESCOPOMANUT, DTINCLUSAO)
              VALUES  (:NRRELESCORDEMSERV, :NRORDEMSERV, :NRESCOPOMANUT, SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelEscopoOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_RELESCORDEMSERV(
                    NRRELESCORDEMSERV,  NRORDEMSERV,    NRESCOPOMANUT,  DTINCLUSAO
                )
                VALUES(
                    :NRRELESCORDEMSERV, :NRORDEMSERV,   :NRESCOPOMANUT, SYSDATE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelSolicitacaoOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_RELSOLICITAORDEMSERV(
                    NRRELSOLICITAORDEMSERV,     NRSOLICITACAOSERV,  NRORDEMSERV,
                    IDATIVO,                    DTINCLUSAO
                )
                VALUES(
                    :NRRELSOLICITAORDEMSERV,    :NRSOLICITACAOSERV, :NRORDEMSERV,
                    'S',                        SYSDATE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }
}
