<?php

namespace FM\Service;

use FM\Factory\Fm070000ParametrosSensorFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class Fm070000ParametrosSensorService
{
    protected $connection;
    protected $Fm070000ParametrosSensorFactory;
    protected $environment;
    protected $newCode;

    public function __construct(Fm070000ParametrosSensorFactory $Fm070000ParametrosSensorFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->Fm070000ParametrosSensorFactory            = $Fm070000ParametrosSensorFactory;
        $this->environment               = $environment;
        $this->newCode                   = $newCode;
        $this->connection                = $entityManager->getConnection();
    }

    public function getParamLeitContPessoas()
    {
        return $this->Fm070000ParametrosSensorFactory->getParamLeitContPessoas();
    }

    public function inserirSensor($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            if ($parametros['NRPARAMLEITCONTPESSOAS'] == null) {
                $nrparamleitcontpessoas = $this->newCode->getCodeByTableName("FCM_PARAMLEITCONTPESSOAS", $nrorg, 1, 10, true);
                $params = array(
                    "NRPARAMLEITCONTPESSOAS"     => $nrparamleitcontpessoas,
                    "CDCLIENTE"     => $parametros['CDCLIENTE'],
                    "IDTIPOSENSOR"     => $parametros['IDTIPOSENSOR'],
                    "IDMACDEVICE" => $parametros['IDMACDEVICE'],
                    "IDSENSOR" => $parametros['IDSENSOR'],
                    "NRQTPESSOASALERT" => $parametros['NRQTPESSOASALERT'],
                    "VRDISTANCIAMINLEIT" => $parametros['VRDISTANCIAMINLEIT'],
                    "VRDISTANCIAMAXLEIT" => $parametros['VRDISTANCIAMAXLEIT'],
                    "VRDISTANCIAMEDLEIT" => $parametros['VRDISTANCIAMEDLEIT'],
                    "QTMATRIZLEITURA" => $parametros['QTMATRIZLEITURA'],
                    "QTTEMPRECONHECIMENTO" => $parametros['QTTEMPRECONHECIMENTO'],
                    "IDMODODISTANCIA" => $parametros['IDMODODISTANCIA'],
                    "QTTEMPOREPETLEIT" => $parametros['QTTEMPOREPETLEIT'],
                    "HRINICIODIA" => $parametros['HRINICIODIA'],
                    "HRFINALDIA" => $parametros['HRFINALDIA'],
                    "IDATIVO"        => 'S',
                    "NRLOCALCLIENTE" => $parametros['NRLOCALCLIENTE'],
                    "DSSENSOR" => $parametros['DSSENSOR'],
                    "IDMONITORAMENTO" => 'N',
                    "NRSEQLEITURA" => null,
                    "QTPESSOAS" => 0,
                    "IPATUALIZASENSORES" => $parametros['IPATUALIZASENSORES'],
                );
                $this->Fm070000ParametrosSensorFactory->inserirSensor($params);
            } else {
                $params = array(
                    "NRPARAMLEITCONTPESSOAS"     => $parametros['NRPARAMLEITCONTPESSOAS'],
                    "CDCLIENTE"     => $parametros['CDCLIENTE'],
                    "IDTIPOSENSOR"     => $parametros['IDTIPOSENSOR'],
                    "IDMACDEVICE" => $parametros['IDMACDEVICE'],
                    "IDSENSOR" => $parametros['IDSENSOR'],
                    "NRQTPESSOASALERT" => $parametros['NRQTPESSOASALERT'],
                    "VRDISTANCIAMINLEIT" => $parametros['VRDISTANCIAMINLEIT'],
                    "VRDISTANCIAMAXLEIT" => $parametros['VRDISTANCIAMAXLEIT'],
                    "VRDISTANCIAMEDLEIT" => $parametros['VRDISTANCIAMEDLEIT'],
                    "QTMATRIZLEITURA" => $parametros['QTMATRIZLEITURA'],
                    "QTTEMPRECONHECIMENTO" => $parametros['QTTEMPRECONHECIMENTO'],
                    "IDMODODISTANCIA" => $parametros['IDMODODISTANCIA'],
                    "QTTEMPOREPETLEIT" => $parametros['QTTEMPOREPETLEIT'],
                    "HRINICIODIA" => $parametros['HRINICIODIA'],
                    "HRFINALDIA" => $parametros['HRFINALDIA'],
                    "IDATIVO"        => 'S',
                    "NRLOCALCLIENTE" => $parametros['NRLOCALCLIENTE'],
                    "DSSENSOR" => $parametros['DSSENSOR'],
                    "IDMONITORAMENTO" => 'N',
                    "NRSEQLEITURA" => null,
                    "QTPESSOAS" => 0,
                    "IPATUALIZASENSORES" => $parametros['IPATUALIZASENSORES'],
                );
                $this->Fm070000ParametrosSensorFactory->atualizarSensor($params);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deletarParametrosSensor($parametros)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($parametros as $chave => $linha) {
                $dataParam = array("NRPARAMLEITCONTPESSOAS" => $linha["NRPARAMLEITCONTPESSOAS"],);
                $this->Fm070000ParametrosSensorFactory->deletarParametrosSensor($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
